@@ -1,4 +1,4 @@
-# Parser for Cleargrass or Qingping BLE advertisements
+"""Parser for Cleargrass or Qingping BLE advertisements"""
 import logging
 from struct import unpack
 
@@ -6,9 +6,9 @@ _LOGGER = logging.getLogger(__name__)
 
 
 def parse_qingping(self, data, source_mac, rssi):
-    # check for adstruc length
+    """Qingping parser"""
     msg_length = len(data)
-    if msg_length > 12 and data[4] == 0x08:
+    if msg_length > 12 and data[4] in [0x08, 0x48]:
         firmware = "Qingping"
         device_id = data[5]
         if device_id == 0x01:
@@ -17,6 +17,8 @@ def parse_qingping(self, data, source_mac, rssi):
             device_type = "CGG1"
         elif device_id == 0x09:
             device_type = "CGP1W"
+        elif device_id == 0x12:
+            device_type = "CGPR1"
         elif device_id == 0x0C:
             device_type = "CGD1"
         else:
@@ -25,7 +27,10 @@ def parse_qingping(self, data, source_mac, rssi):
         qingping_mac_reversed = data[6:12]
         qingping_mac = qingping_mac_reversed[::-1]
 
-        result = {"rssi": rssi}
+        result = {
+            "rssi": rssi,
+            "packet": "no packet id",
+        }
         xdata_point = 14
         while xdata_point < msg_length:
             xdata_id = data[xdata_point - 2]
@@ -40,6 +45,25 @@ def parse_qingping(self, data, source_mac, rssi):
                 elif xdata_id == 0x07 and xdata_size == 2:
                     (pres,) = unpack("<H", data[xdata_point:xdata_point + xdata_size])
                     result.update({"pressure": pres / 10})
+                elif xdata_id == 0x08 and xdata_size == 4:
+                    (motion, illuminance_1, illuminance_2) = unpack(
+                        "<BHB", data[xdata_point:xdata_point + xdata_size]
+                    )
+                    result.update({
+                        "motion": motion,
+                        "illuminance": illuminance_1 + illuminance_2,
+                    })
+                    if motion:
+                        result.update({"motion timer": 1})
+                elif xdata_id == 0x09 and xdata_size == 4:
+                    (illuminance,) = unpack("<I", data[xdata_point:xdata_point + xdata_size])
+                    result.update({"illuminance": illuminance})
+                elif xdata_id == 0x11 and xdata_size == 1:
+                    light = data[xdata_point]
+                    result.update({"light": light})
+                elif xdata_id == 0x0F and xdata_size == 1:
+                    packet_id = data[xdata_point]
+                    result.update({"packet": packet_id})
                 else:
                     _LOGGER.debug(
                         "Unknown data received from Qingping device: %s",
@@ -72,7 +96,6 @@ def parse_qingping(self, data, source_mac, rssi):
         "rssi": rssi,
         "mac": ''.join('{:02X}'.format(x) for x in qingping_mac[:]),
         "type": device_type,
-        "packet": "no packet id",
         "firmware": firmware,
         "data": True
     })
@@ -80,4 +103,5 @@ def parse_qingping(self, data, source_mac, rssi):
 
 
 def to_mac(addr: int):
+    """Return formatted MAC address"""
     return ':'.join('{:02x}'.format(x) for x in addr).upper()
